/*
 * This file is part of the rtap localisation project.
 */

#ifndef _RTAPAGGREGATE_H
#define _RTAPAGGREGATE_H

#include <postgresql/libpq-fe.h>
#include "../../libowlps/owlps.h"
#include <confuse.h>

#define DEBUG

//#define TIMESTAMP // Décommenter pour afficher le timestamp à chaque ligne du fichier de sortie.


/* Arguments & configuration du programme */
#define OPTIONS "a:c:f:H:i:k:l:N:o:p:P:U:" // Chaîne pour getopt
#define DEFAULT_CONFIG_FILE "/usr/local/etc/owlps/owlps-aggregator.conf"
#define DEFAULT_AGGREGATE_TIMEOUT 1500 // Timeout d'agrégation (en millisecondes)
#define DEFAULT_KEEP_TIMEOUT 3000 // Temps que l'on conserve les données dans la liste (en millisecondes)
#define DEFAULT_CHECK_INTERVAL 500000 // Temps entre deux vérifications de la liste (en micro-secondes)
#define POSITIONER_DEFAULT_IP "127.0.0.1"
#define DEFAULT_DB_HOST "127.0.0.1"
#define DEFAULT_DB_NAME "owlps_db"
#define DEFAULT_DB_USER "owlps"
#define DEFAULT_DB_PASS "owlps"

/* Codes d'erreur */
#define ERR_NO_MESSAGE_RECEIVED 1 // Erreur lors de la lecture sur la socket
#define ERR_CREATING_SOCKET 2 // Erreur lors de la création de la socket d'écoute
#define ERR_BAD_USAGE 3 // Mauvais appel au programme
#define ERR_PARSING_CONFIG_FILE 4 // Erreur lors de la lecture du fichier de configuration


/* Liste chaînée des informations concernant un couple MAC / séquence */
typedef struct _couple_info_list
{
  unsigned char ap_mac_addr_bytes[6] ; // Adresse MAC de l'AP émetteur de l'info en octets
  unsigned char antenna_signal_dbm ; // Puissance du signal reçu par l'AP du mobile
  struct _couple_info_list *next ;
} couple_info_list ;


/* Liste chaînée des couples MAC / séquence */
typedef struct _couple_list
{
  /* Numéro de séquence de la demande de localisation du mobile */
  unsigned char mobile_mac_addr_bytes[6] ; // Adresse MAC du mobile en octets
  struct timeval request_time ; //  Heure de la demande sur le mobile

  /* Données pour la calibration */
  float x_position ;
  float y_position ;
  float z_position ;
  DIRECTION direction ; // Orientation de la demande de localisation

  /* Autres données */
  struct timeval start_time ; // Heure d'arrivée du premier paquet du couple sur le serveur d'agrégation
  couple_info_list *info ; // Liste des informations pour ce couple
  struct _couple_list *next ;
} couple_list ;


/* En-têtes de fonctions */
void got_couple_info(couple_list **couples, couple_message message) ;
void free_couple_list(couple_list **couples) ;
#ifdef DEBUG
void print_couple_list(couple_list *couples) ;
void print_couple_info(couple_info_list *info) ;
#endif // DEBUG
void* monitor_couple_list(couple_list **couples) ;
int Db_write_request(couple_message message, BOOL ref) ;
int DbAccessPoint(couple_message message, PGconn *conn);
int DbMobile(couple_message message, PGconn *conn);
int DbRefPoint(couple_message message, PGconn *conn);
int DbRequest(couple_message message, PGconn *conn, int id_mobile, int *id_point, BOOL ref);
int DbPacket(couple_message message, PGconn *conn, int id_ap, int id_request, BOOL ref);
void print_usage(char *prog) ;

#endif
